@@ -64,8 +64,8 @@ class OptionChain:
         return symbols
 
     def init_dataframe(self):
-        df = pd.DataFrame(columns=['symbols' , 'strike' , 'type' , 'ltp' , 'ltt' ,'volume' , 'oi' , 'prev_oi' , 'oi_change' ,
-                                    'oi_change_perc' , 'bid' , 'bid_qty' , 'ask' , 'ask_qty'])
+        df = pd.DataFrame(columns=['symbols' , 'strike' , 'type' , 'ltp' , 'ltt' , 'ltq' , 'volume' , 'price_change' , 'price_change_perc', 'oi' , 
+                                    'prev_oi' , 'oi_change' , 'oi_change_perc' , 'bid' , 'bid_qty' , 'ask' , 'ask_qty'])
         df.symbols = self.option_symbols
         df.strike = df.symbols.apply(lambda x: str(re.search(r"\d+(\.\d+)?", x ).group(0) )[6:])
         df.type = df.symbols.apply(lambda x: re.findall('\D+' , x )[-1] )
@@ -73,27 +73,35 @@ class OptionChain:
         df.sort_values(by=['strike' , 'type'] , inplace=True)
         return df
 
-    def update_chain(self ,td_obj , symb_ids ):
-        self.chain_status = True
+    def initial_update_from_touchline(self , td_obj , symb_ids):
         prev_tick_obj = {}
         time.sleep(1)
         for symb_id in symb_ids:
             prev_tick_obj[symb_id] = deepcopy(td_obj.live_data[symb_id])
             try:
                 oi_change_perc = round((td_obj.touchline_data[symb_id].oi - td_obj.touchline_data[symb_id].prev_oi)*100 / (td_obj.touchline_data[symb_id].prev_oi) , 2 )
+                price_change_perc = round((td_obj.touchline_data[symb_id].ltp - td_obj.touchline_data[symb_id].prev_close)*100 / (td_obj.touchline_data[symb_id].prev_oi) , 4 )
             except (ZeroDivisionError , TypeError):
                 oi_change_perc = 0
+                price_change_perc = 0
             try:
-                self.chain_dataframe.loc[td_obj.touchline_data[symb_id].symbol , ['ltp' , 'ltt'  , 'volume' ,  'oi' , 'prev_oi', 'oi_change' ,
-                                                                                    'oi_change_perc','bid' , 'bid_qty' , 'ask' , 'ask_qty' ]] = [
-                    td_obj.touchline_data[symb_id].ltp, td_obj.touchline_data[symb_id].timestamp , td_obj.touchline_data[symb_id].ttq ,
-                    td_obj.touchline_data[symb_id].oi, td_obj.touchline_data[symb_id].prev_oi , td_obj.touchline_data[symb_id].oi - td_obj.touchline_data[symb_id].prev_oi ,
-                    oi_change_perc , td_obj.touchline_data[symb_id].best_bid_price, td_obj.touchline_data[symb_id].best_bid_qty,
-                    td_obj.touchline_data[symb_id].best_ask_price, td_obj.touchline_data[symb_id].best_ask_qty]
+                self.chain_dataframe.loc[td_obj.touchline_data[symb_id].symbol , ['ltp' , 'ltt'  , 'ltq' , 'volume' ,'price_change' , 'price_change_perc' , 'oi' , 
+                                                        'prev_oi', 'oi_change' , 'oi_change_perc','bid' , 'bid_qty' , 'ask' , 'ask_qty' ]] = [
+                    td_obj.touchline_data[symb_id].ltp, td_obj.touchline_data[symb_id].timestamp , td_obj.touchline_data[symb_id].ltq , 
+                    td_obj.touchline_data[symb_id].ttq ,td_obj.touchline_data[symb_id].ltp - td_obj.touchline_data[symb_id].prev_close , 
+                    price_change_perc , td_obj.touchline_data[symb_id].oi, td_obj.touchline_data[symb_id].prev_oi , 
+                    td_obj.touchline_data[symb_id].oi - td_obj.touchline_data[symb_id].prev_oi ,oi_change_perc , 
+                    td_obj.touchline_data[symb_id].best_bid_price, td_obj.touchline_data[symb_id].best_bid_qty,
+                    td_obj.touchline_data[symb_id].best_ask_price, td_obj.touchline_data[symb_id].best_ask_qty 
+                ]
             except TypeError:
-                self.chain_dataframe.loc[td_obj.touchline_data[symb_id].symbol , ['ltp' , 'ltt'  , 'volume' ,  'oi' , 'prev_oi', 'oi_change' ,
-                                                                                    'oi_change_perc','bid' , 'bid_qty' , 'ask' , 'ask_qty' ]] =[ np.NaN for i in range(0,11)]
+                self.chain_dataframe.loc[td_obj.touchline_data[symb_id].symbol , ['ltp' , 'ltt' ,'ltq' , 'volume' , 'price_change' , 'price_change_perc',  'oi' , 'prev_oi', 'oi_change' ,
+                                                            'oi_change_perc','bid' , 'bid_qty' , 'ask' , 'ask_qty' ]] =[ np.NaN for i in range(14)]
+        return prev_tick_obj
 
+    def update_chain(self ,td_obj , symb_ids ):
+        self.chain_status = True
+        prev_tick_obj = self.initial_update_from_touchline(td_obj, symb_ids)
         while self.chain_status:
             for symb_id in symb_ids:
                 if not td_obj.live_data[symb_id] == prev_tick_obj[symb_id]:
@@ -102,15 +110,18 @@ class OptionChain:
                         ask , ask_qnty = td_obj.live_data[symb_id].best_ask_price, td_obj.live_data[symb_id].best_ask_qty
                     except Exception as e:
                         bid, bid_qnty, ask, ask_qnty = 0, 0, 0, 0
-                    self.chain_dataframe.loc[td_obj.live_data[symb_id].symbol , ['ltt' , 'volume' ,  'oi' ,'prev_oi' , 'oi_change' ,
+                    self.chain_dataframe.loc[td_obj.live_data[symb_id].symbol , ['ltt' , 'volume' , 'price_change','price_change_perc', 'oi' ,'prev_oi' , 'oi_change' ,
                                                                                 'oi_change_perc' ,'bid' , 'bid_qty' , 'ask' , 'ask_qty' ]] = [
-                        td_obj.live_data[symb_id].timestamp ,td_obj.live_data[symb_id].ttq , td_obj.live_data[symb_id].oi,
-                        td_obj.live_data[symb_id].prev_day_oi , td_obj.live_data[symb_id].oi_change ,
-                        round(td_obj.live_data[symb_id].to_dict()['oi_change_perc'] ,2 ) ,bid, bid_qnty, ask, ask_qnty]
+                        td_obj.live_data[symb_id].timestamp , td_obj.live_data[symb_id].ttq ,td_obj.live_data[symb_id].change, 
+                        round(td_obj.live_data[symb_id].change_perc , 2) ,td_obj.live_data[symb_id].oi, td_obj.live_data[symb_id].prev_day_oi ,
+                        td_obj.live_data[symb_id].oi_change ,round(td_obj.live_data[symb_id].to_dict()['oi_change_perc'] ,2 ) ,bid, bid_qnty, ask, ask_qnty 
+                    ]
                     if 'ltp' in td_obj.live_data[symb_id].__dict__.keys():
-                        self.chain_dataframe.loc[td_obj.live_data[symb_id].symbol , ['ltp'] ] = td_obj.live_data[symb_id].ltp
+                        self.chain_dataframe.loc[td_obj.live_data[symb_id].symbol , ['ltp'] ] = td_obj.live_data[symb_id].ltp    
+                        self.chain_dataframe.loc[td_obj.live_data[symb_id].symbol , ['ltq'] ] = td_obj.live_data[symb_id].ltq if td_obj.live_data[symb_id].ltq is not None else 0
                     else:
                         self.chain_dataframe.loc[td_obj.live_data[symb_id].symbol , ['ltp'] ] = td_obj.live_data[symb_id].close
+                        self.chain_dataframe.loc[td_obj.live_data[symb_id].symbol , ['ltq'] ] = td_obj.live_data[symb_id].volume if td_obj.live_data[symb_id].volume is not None else 0
                     prev_tick_obj[symb_id] = deepcopy(td_obj.live_data[symb_id])
             time.sleep(0.005)
             if not(self.market_open_post_hours) and dt.now().time() > OptionChain.close_time :
@@ -118,10 +129,11 @@ class OptionChain:
         # print('exited chain thread')
 
     def get_option_chain(self):
-        df = self.chain_dataframe
+        df = self.chain_dataframe.copy()
+        # df.dropna(inplace=True)
+        df = df.astype({'ltq' : int , 'volume': int })
         if not self.bid_ask:
             df = df.drop(['bid' , 'bid_qty' , 'ask' , 'ask_qty'] , axis= 1  )
-        df.dropna(inplace=True)
         return df
 
     def stop_option_chain(self):
